@@ -470,8 +470,8 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Microsoft Windows 3.X DIB image file.
   */
-  dib_info.width=(short) ReadBlobLSBLong(image);
-  dib_info.height=(short) ReadBlobLSBLong(image);
+  dib_info.width=(magick_int32_t) ReadBlobLSBLong(image);
+  dib_info.height=(magick_int32_t) ReadBlobLSBLong(image);
   dib_info.planes=ReadBlobLSBShort(image);
   dib_info.bits_per_pixel=ReadBlobLSBShort(image);
   dib_info.compression=ReadBlobLSBLong(image);
@@ -487,6 +487,10 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       dib_info.green_mask=ReadBlobLSBShort(image);
       dib_info.blue_mask=ReadBlobLSBShort(image);
     }
+  if (dib_info.width <= 0)
+      ThrowReaderException(CorruptImageWarning,NegativeOrZeroImageSize,image);
+  if (dib_info.height == 0)
+      ThrowReaderException(CorruptImageWarning,NegativeOrZeroImageSize,image);
   image->matte=dib_info.bits_per_pixel == 32;
   image->columns=dib_info.width;
   image->rows=AbsoluteValue(dib_info.height);
@@ -528,7 +532,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       if (!AllocateImageColormap(image,image->colors))
         ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
-      dib_colormap=MagickAllocateMemory(unsigned char *,4*image->colors);
+      dib_colormap=MagickAllocateMemoryElements(unsigned char *,image->colors,4);
       if (dib_colormap == (unsigned char *) NULL)
         ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
       packet_size=4;
@@ -551,8 +555,9 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     dib_info.bits_per_pixel<<=1;
   bytes_per_line=4*((image->columns*dib_info.bits_per_pixel+31)/32);
   length=bytes_per_line*image->rows;
-  pixels=MagickAllocateMemory(unsigned char *,
-    Max(bytes_per_line,image->columns+1)*image->rows);
+  pixels=MagickAllocateMemoryElements(unsigned char *,
+                                      image->rows,
+                                      Max(bytes_per_line,image->columns+1));
   if (pixels == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
   if ((dib_info.compression == 0) || (dib_info.compression == 3))
