@@ -45,6 +45,7 @@
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/quantize.h"
 #include "magick/resize.h"
 #include "magick/tempfile.h"
 #include "magick/utility.h"
@@ -370,7 +371,6 @@ static unsigned int TIFFErrors(const char *module,const char *format,
     message[MaxTextExtent];
 
   errno=0;
-  (void) vsprintf(message,format,warning);
   (void) vsnprintf(message,MaxTextExtent-2,format,warning);
   message[MaxTextExtent-2]='\0';
   (void) strcat(message,".");
@@ -1383,8 +1383,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
         break;
       }
     }
-    if (image->depth > QuantumDepth)
-      image->depth=QuantumDepth;
+
     /*
       Proceed to next image.
     */
@@ -1459,6 +1458,7 @@ ModuleExport void RegisterTIFFImage(void)
 #endif
 
   entry=SetMagickInfo("PTIF");
+  entry->thread_support=False; /* libtiff uses libjpeg which is not thread safe */
   entry->decoder=(DecoderHandler) ReadTIFFImage;
   entry->encoder=(EncoderHandler) WritePTIFImage;
   entry->adjoin=False;
@@ -1467,6 +1467,7 @@ ModuleExport void RegisterTIFFImage(void)
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("TIF");
+  entry->thread_support=False; /* libtiff uses libjpeg which is not thread safe */
   entry->decoder=(DecoderHandler) ReadTIFFImage;
   entry->encoder=(EncoderHandler) WriteTIFFImage;
   entry->description=AcquireString(TIFFDescription);
@@ -1476,6 +1477,7 @@ ModuleExport void RegisterTIFFImage(void)
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("TIFF");
+  entry->thread_support=False; /* libtiff uses libjpeg which is not thread safe */
   entry->decoder=(DecoderHandler) ReadTIFFImage;
   entry->encoder=(EncoderHandler) WriteTIFFImage;
   entry->magick=(MagickHandler) IsTIFF;
@@ -1554,24 +1556,39 @@ static unsigned int WritePTIFImage(const ImageInfo *image_info,Image *image)
   ImageInfo
     *clone_info;
 
+  FilterTypes
+    filter;
+
   unsigned int
     status;
 
   /*
     Create pyramid-encoded TIFF image.
   */
+  assert(image_info != (const ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  filter=TriangleFilter;
+  if (image->is_monochrome)
+    filter=PointFilter;
   pyramid_image=CloneImage(image,0,0,True,&image->exception);
   if (pyramid_image == (Image *) NULL)
     ThrowWriterException2(FileOpenError,image->exception.reason,image);
   do
-  {
-    pyramid_image->next=ResizeImage(image,pyramid_image->columns/2,
-      pyramid_image->rows/2,TriangleFilter,1.0,&image->exception);
-    if (pyramid_image->next == (Image *) NULL)
-      ThrowWriterException2(FileOpenError,image->exception.reason,image);
-    pyramid_image->next->previous=pyramid_image;
-    pyramid_image=pyramid_image->next;
-  } while ((pyramid_image->columns > 64) && (pyramid_image->rows > 64));
+    {
+      pyramid_image->next=ResizeImage(image,pyramid_image->columns/2,
+                                      pyramid_image->rows/2,TriangleFilter,
+                                      1.0,&image->exception);
+      if (pyramid_image->next == (Image *) NULL)
+        ThrowWriterException2(FileOpenError,image->exception.reason,image);
+      if ((!image->is_monochrome) && (image->storage_class == PseudoClass))
+        (void) MapImage(pyramid_image->next,image,False);
+      pyramid_image->next->x_resolution=pyramid_image->x_resolution/2;
+      pyramid_image->next->y_resolution=pyramid_image->y_resolution/2;
+      pyramid_image->next->previous=pyramid_image;
+      pyramid_image=pyramid_image->next;
+    } while ((pyramid_image->columns > 64) && (pyramid_image->rows > 64));
   while (pyramid_image->previous != (Image *) NULL)
     pyramid_image=pyramid_image->previous;
   /*
@@ -1579,6 +1596,13 @@ static unsigned int WritePTIFImage(const ImageInfo *image_info,Image *image)
   */
   clone_info=CloneImageInfo(image_info);
   clone_info->adjoin=True;
+  strlcpy(clone_info->magick,"TIFF",MaxTextExtent);
+  strlcpy(image->magick,"TIFF",MaxTextExtent);
+  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                        "Invoking \"%.1024s\" encoder, monochrome=%s, grayscale=%s",
+                        "TIFF",
+                        (image->is_monochrome != MagickFalse ? "True" : "False"),
+                        (image->is_grayscale != MagickFalse ? "True" : "False"));
   status=WriteTIFFImage(clone_info,pyramid_image);
   DestroyImageList(pyramid_image);
   DestroyImageInfo(clone_info);
@@ -1802,7 +1826,8 @@ static int32 TIFFWritePixels(TIFF *tiff,tdata_t scanline,unsigned long row,
 static MagickPassFail WriteTIFFImage(const ImageInfo *image_info,Image *image)
 {
   char
-    filename[MaxTextExtent];
+    filename[MaxTextExtent],
+    open_flags[MaxTextExtent];
 
   const ImageAttribute
     *attribute;
@@ -1826,6 +1851,7 @@ static MagickPassFail WriteTIFFImage(const ImageInfo *image_info,Image *image)
   uint16
     bits_per_sample,
     compress_tag,
+    fill_order,
     photometric,
     samples_per_pixel;
 
@@ -1857,11 +1883,11 @@ static MagickPassFail WriteTIFFImage(const ImageInfo *image_info,Image *image)
   (void) TIFFSetErrorHandler((TIFFErrorHandler) TIFFErrors);
   (void) TIFFSetWarningHandler((TIFFErrorHandler) TIFFWarnings);
   (void) strncpy(filename,image->filename,MaxTextExtent-1);
-  if (!(GetBlobFileHandle(image)))
+  if (!(GetBlobFileHandle(image)) || !(BlobIsSeekable(image)))
     {
       /*
-        If output is not to a stdio file descriptor, then use a
-        temporary file for the output so that it is.
+        If output is not to a stdio file descriptor or not seekable,
+        then use a temporary file for the output so that it is.
       */
       filename_is_temporary=True;
       if(!AcquireTemporaryFileName(filename))
@@ -1872,9 +1898,44 @@ static MagickPassFail WriteTIFFImage(const ImageInfo *image_info,Image *image)
   /*
     Open TIFF file
 
-    'w'          open for write
+    'w'  open for write
+    'l'  force little-endian byte order
+    'b'  force big-endian byte order
+    'L'  force LSB to MSB bit order (weird)
+    'B'  force MSB to LSB bit order (normal)
+    '8'  64-bit offsets (BigTIFF)
   */
-  tiff=TIFFOpen(filename,"w");
+  (void) strlcpy(open_flags, "w", sizeof(open_flags));
+  switch (image_info->endian)
+    {
+    case LSBEndian:
+      (void) strlcat(open_flags, "l", sizeof(open_flags));
+      if (logging)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "Using little endian byte order");
+      break;
+    case MSBEndian:
+      (void) strlcat(open_flags, "b", sizeof(open_flags));
+      if (logging)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "Using big endian byte order");
+      break;
+    default:
+    case UndefinedEndian:
+      {
+        /* Default is native byte order */
+        if (logging)
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                "Using native endian byte order");
+      }
+    }
+
+  if (logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "Opening TIFF file \"%s\" using open flags \"%s\".",
+                          filename,open_flags);
+
+  tiff=TIFFOpen(filename,open_flags);
   if (tiff == (TIFF *) NULL)
     {
       if (filename_is_temporary)
@@ -1900,9 +1961,16 @@ static MagickPassFail WriteTIFFImage(const ImageInfo *image_info,Image *image)
     depth=image->depth;
     bits_per_sample=8;
     compress_tag=COMPRESSION_NONE;
+    fill_order=FILLORDER_MSB2LSB;
     switch (image->compression)
     {
 #ifdef CCITT_SUPPORT
+      /*
+        Note that RFC 2301 recommends using LSB2MSB fill order for FAX
+        since it is the transmission order used on the wire for FAX.
+        However, it also states that all conforming readers should be
+        able to read data in both bit orders.
+      */
       case FaxCompression:
       {
         (void) SetImageType(image,BilevelType);
@@ -1910,6 +1978,7 @@ static MagickPassFail WriteTIFFImage(const ImageInfo *image_info,Image *image)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "Set image type to BilevelType");
         compress_tag=COMPRESSION_CCITTFAX3;
+        fill_order=FILLORDER_LSB2MSB;
         break;
       }
       case Group4Compression:
@@ -1919,6 +1988,7 @@ static MagickPassFail WriteTIFFImage(const ImageInfo *image_info,Image *image)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "Set image type to BilevelType");
         compress_tag=COMPRESSION_CCITTFAX4;
+        fill_order=FILLORDER_LSB2MSB;
         break;
       }
 #endif
@@ -2113,6 +2183,18 @@ static MagickPassFail WriteTIFFImage(const ImageInfo *image_info,Image *image)
             }
         }
 
+      /*
+        Fill order
+      */
+      value=AccessDefinition(image_info,"tiff","fill-order");
+      if (value)
+        {
+          if (LocaleNCompare(value,"msb2lsb",3) == 0)
+            fill_order=FILLORDER_MSB2LSB;
+          else if (LocaleNCompare(value,"lsb2msb",3) == 0)
+            fill_order=FILLORDER_LSB2MSB;
+        }
+
       /*
         Samples per pixel
       */
@@ -2140,37 +2222,14 @@ static MagickPassFail WriteTIFFImage(const ImageInfo *image_info,Image *image)
 
     (void) TIFFSetField(tiff,TIFFTAG_PHOTOMETRIC,photometric);
     (void) TIFFSetField(tiff,TIFFTAG_COMPRESSION,compress_tag);
-    switch (image_info->endian)
-    {
-      case LSBEndian:
-        (void) TIFFSetField(tiff,TIFFTAG_FILLORDER,FILLORDER_LSB2MSB);
-        if (logging)
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-             "Using LSB2MSB fillorder");
-        break;
-      case MSBEndian:
-        (void) TIFFSetField(tiff,TIFFTAG_FILLORDER,FILLORDER_MSB2LSB);
-        if (logging)
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-             "Using MSB2LSB fillorder");
-        break;
-      default:
-      case UndefinedEndian:
-      {
-#if defined(HOST_BIGENDIAN)
-        (void) TIFFSetField(tiff,TIFFTAG_FILLORDER,FILLORDER_MSB2LSB);
-        if (logging)
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-             "Using MSB2LSB fillorder");
-#else
-        (void) TIFFSetField(tiff,TIFFTAG_FILLORDER,FILLORDER_LSB2MSB);
-        if (logging)
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-             "Using LSB2MSB fillorder");
-#endif
-        break;
-      }
-    }
+    if (FILLORDER_MSB2LSB != fill_order)
+      (void) TIFFSetField(tiff,TIFFTAG_FILLORDER,fill_order);
+    if (logging)
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "Using %s bit fill order",
+                            (fill_order == FILLORDER_MSB2LSB ? "MSB2LSB" :
+                             (fill_order == FILLORDER_LSB2MSB ? "LSB2MSB" :
+                              "undefined")));
     (void) TIFFSetField(tiff,TIFFTAG_ORIENTATION,ORIENTATION_TOPLEFT);
     (void) TIFFSetField(tiff,TIFFTAG_PLANARCONFIG,PLANARCONFIG_CONTIG);
     if (photometric == PHOTOMETRIC_RGB)
@@ -2199,8 +2258,38 @@ static MagickPassFail WriteTIFFImage(const ImageInfo *image_info,Image *image)
         (void) TIFFSetField(tiff,TIFFTAG_ZIPQUALITY,9);
         break;
       }
+    case COMPRESSION_CCITTFAX3:
+      {
+        /*
+          Set Group 3 Options.  Group 3 options are arranged as 32 flag bits.
+          Specify byte-aligned EOL padding option.
+          
+          Group3Options = 4,5. LONG. Data may be one- or
+          two-dimensional, but EOLs must be
+          byte-aligned. Uncompressed data is not allowed.
+          
+          bit 0 = 0 for 1-Dimensional, 1 for 2-Dimensional
+          
+          bit 1 = must be 0 (uncompressed data not allowed)
+          
+          bit 2 = 1 for byte-aligned EOLs
+          
+        */
+        (void) TIFFSetField(tiff,TIFFTAG_GROUP3OPTIONS,4);
+        
+        /*
+          It is recommended (but not required) to output FAX as
+          one strip.
+        */
+        (void) TIFFSetField(tiff,TIFFTAG_ROWSPERSTRIP,image->rows);
+        break;
+      }
       case COMPRESSION_CCITTFAX4:
       {
+        /*
+          It is recommended (but not required) to output FAX as
+          one strip.
+        */
         (void) TIFFSetField(tiff,TIFFTAG_ROWSPERSTRIP,image->rows);
         break;
       }
@@ -2237,6 +2326,14 @@ static MagickPassFail WriteTIFFImage(const ImageInfo *image_info,Image *image)
           units=RESUNIT_INCH;
         if (image->units == PixelsPerCentimeterResolution)
           units=RESUNIT_CENTIMETER;
+        if (logging)
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                "Resolution %gx%g pixels%s",
+                                image->x_resolution, image->y_resolution,
+                                (units == RESUNIT_NONE ? " (undefined units)" :
+                                 (units == RESUNIT_INCH ? " per inch" :
+                                  (units == RESUNIT_CENTIMETER ? " per centimeter"
+                                   : "BAD VALUE"))));
         (void) TIFFSetField(tiff,TIFFTAG_RESOLUTIONUNIT,(uint16) units);
         (void) TIFFSetField(tiff,TIFFTAG_XRESOLUTION,image->x_resolution);
         (void) TIFFSetField(tiff,TIFFTAG_YRESOLUTION,image->y_resolution);
@@ -2274,12 +2371,25 @@ static MagickPassFail WriteTIFFImage(const ImageInfo *image_info,Image *image)
       WriteNewsProfile(tiff,TIFFTAG_RICHTIFFIPTC,image);
 #endif
 #endif
-    if (image_info->adjoin && (GetImageListLength(image) > 1))
       {
-        (void) TIFFSetField(tiff,TIFFTAG_SUBFILETYPE,FILETYPE_PAGE);
-        if (image->scene != 0)
-          (void) TIFFSetField(tiff,TIFFTAG_PAGENUMBER,(unsigned short)
-            image->scene,GetImageListLength(image));
+        /*
+          Page and Page number tags.  Page is the current page number
+          (0 based) and pages is the total number of pages.*/
+
+        uint16
+          page,
+          pages;
+
+        page=(uint16) scene;
+        pages=GetImageListLength(image);
+
+        if (image_info->adjoin && pages > 1)
+        {
+          /* SubFileType = 2. LONG. The value 2 identifies a single page of a multi-page image. */
+          (void) TIFFSetField(tiff,TIFFTAG_SUBFILETYPE,FILETYPE_PAGE);
+        }
+
+        (void) TIFFSetField(tiff,TIFFTAG_PAGENUMBER,page,pages);
       }
     attribute=GetImageAttribute(image,"artist");
     if (attribute != (const ImageAttribute *) NULL)
