@@ -47,6 +47,9 @@
 #if defined(WIN32) || defined(__CYGWIN__)
 # include "magick/nt_feature.h"
 #endif
+#if defined(POSIX)
+# include "magick/unix_port.h"
+#endif
 #include "magick/semaphore.h"
 #include "magick/tempfile.h"
 #include "magick/utility.h"
@@ -336,6 +339,50 @@ MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
 %
 %
 */
+static size_t
+UnixShellTextEscape(char *dst, const char *src, const size_t size)
+{
+  size_t
+    length=0;
+
+  char
+    *p;
+
+  const char
+    *q;
+
+  assert(dst != NULL);
+  assert(src != (const char *) NULL);
+  assert(size >= 1);
+
+  /*
+    Copy src to dst within bounds of size-1, while escaping special
+    characters.
+  */
+  for ( p=dst, q=src, length=0 ;
+        (*q != 0) && (length < size-1) ;
+        length++, p++, q++ )
+    {
+      register const char c = *q;
+      if ((c == '\\') ||
+          (c == '`') ||
+          (c == '"') ||
+          (c == '$'))
+        {
+          if (length+1 >= size-1)
+            break;
+          *p = '\\';
+          p++;
+          length++;
+        }
+      *p = c;
+    }
+
+  dst[length]='\0';
+
+  return length;
+}
+
 MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
   const char *decode,const char *encode,ExceptionInfo *exception)
 {
@@ -371,12 +418,12 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
           return(False);
         }
     }
-  (void) strncpy(filename,image->filename,MaxTextExtent-1);
+  (void) strlcpy(filename,image->filename,MaxTextExtent);
   delegate_info=GetDelegateInfo(decode,encode,exception);
   if (delegate_info == (DelegateInfo *) NULL)
     {
       if (temporary_image_filename)
-        LiberateTemporaryFile(image->filename);
+        (void) LiberateTemporaryFile(image->filename);
       (void) ThrowException(exception,DelegateError,NoTagFound,
         decode ? decode : encode);
       return(False);
@@ -391,7 +438,7 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
       if(!AcquireTemporaryFileName(image_info->filename))
         {
           if (temporary_image_filename)
-            LiberateTemporaryFile(image->filename);
+            (void) LiberateTemporaryFile(image->filename);
           (void) ThrowException(exception,FileOpenError,UnableToCreateTemporaryFile,image_info->filename);
           return(False);
         }
@@ -419,7 +466,7 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
         if (!AcquireTemporaryFileName(image_info->unique))
         {
           if (temporary_image_filename)
-            LiberateTemporaryFile(image->filename);
+            (void) LiberateTemporaryFile(image->filename);
           (void) ThrowException(exception,FileOpenError,UnableToCreateTemporaryFile,image_info->unique);
           return(False);
         }
@@ -427,8 +474,8 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
         if (!AcquireTemporaryFileName(image_info->zero))
         {
           if (temporary_image_filename)
-            LiberateTemporaryFile(image->filename);
-          LiberateTemporaryFile(image_info->unique);
+            (void) LiberateTemporaryFile(image->filename);
+          (void) LiberateTemporaryFile(image_info->unique);
           (void) ThrowException(exception,FileOpenError,UnableToCreateTemporaryFile,image_info->zero);
           return(False);
         }
@@ -437,24 +484,24 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
           delegate_info->encode : delegate_info->decode);
         if (magick == (char *) NULL)
           {
-            LiberateTemporaryFile(image_info->unique);
-            LiberateTemporaryFile(image_info->zero);
+            (void) LiberateTemporaryFile(image_info->unique);
+            (void) LiberateTemporaryFile(image_info->zero);
             if (temporary_image_filename)
-              LiberateTemporaryFile(image->filename);
+              (void) LiberateTemporaryFile(image->filename);
             (void) ThrowException(exception,DelegateError,DelegateFailed,
               decode ? decode : encode);
             return(False);
           }
         LocaleUpper(magick);
         clone_info=CloneImageInfo(image_info);
-        (void) strncpy((char *) clone_info->magick,magick,MaxTextExtent-1);
-        (void) strncpy(image->magick,magick,MaxTextExtent-1);
+        (void) strlcpy((char *) clone_info->magick,magick,MaxTextExtent);
+        (void) strlcpy(image->magick,magick,MaxTextExtent);
         MagickFreeMemory(magick);
-        (void) strncpy(filename,image->filename,MaxTextExtent-1);
+        (void) strlcpy(filename,image->filename,MaxTextExtent);
         FormatString(clone_info->filename,"%.1024s:",delegate_info->decode);
         (void) SetImageInfo(clone_info,True,exception);
-        (void) strncpy(clone_info->filename,image_info->filename,
-          MaxTextExtent-1);
+        (void) strlcpy(clone_info->filename,image_info->filename,
+          MaxTextExtent);
         for (p=image; p != (Image *) NULL; p=p->next)
         {
           FormatString(p->filename,"%.1024s:%.1024s",delegate_info->decode,
@@ -462,10 +509,10 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
           status=WriteImage(clone_info,p);
           if (status == False)
             {
-              LiberateTemporaryFile(image_info->unique);
-              LiberateTemporaryFile(image_info->zero);
+              (void) LiberateTemporaryFile(image_info->unique);
+              (void) LiberateTemporaryFile(image_info->zero);
               if (temporary_image_filename)
-                LiberateTemporaryFile(image->filename);
+                (void) LiberateTemporaryFile(image->filename);
               DestroyImageInfo(clone_info);
               (void) ThrowException(exception,DelegateError,DelegateFailed,
                 decode ? decode : encode);
@@ -474,19 +521,19 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
           if (clone_info->adjoin)
             break;
         }
-        LiberateTemporaryFile(image_info->unique);
-        LiberateTemporaryFile(image_info->zero);
+        (void) LiberateTemporaryFile(image_info->unique);
+        (void) LiberateTemporaryFile(image_info->zero);
         DestroyImageInfo(clone_info);
       }
   /*
     Invoke delegate.
   */
-  (void) strncpy(image->filename,filename,MaxTextExtent-1);
+  (void) strlcpy(image->filename,filename,MaxTextExtent);
   commands=StringToList(delegate_info->commands);
   if (commands == (char **) NULL)
     {
       if (temporary_image_filename)
-        LiberateTemporaryFile(image->filename);
+        (void) LiberateTemporaryFile(image->filename);
       (void) ThrowException(exception,ResourceLimitError,MemoryAllocationFailed,decode ? decode : encode);
       return(False);
     }
@@ -506,23 +553,107 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
     if (!AcquireTemporaryFileName(image_info->zero))
     {
       (void) ThrowException(exception,FileOpenError,UnableToCreateTemporaryFile,image_info->zero);
-      LiberateTemporaryFile(image_info->unique);
+      (void) LiberateTemporaryFile(image_info->unique);
       status=False;
       goto error_exit;
     }
-    /* Expand sprintf-style codes in delegate command to command string */
-    command=TranslateText(image_info,image,commands[i]);
-    if (command == (char *) NULL)
-      break;
-    /* Handle commands which should be backgrounded */
-    if (delegate_info->spawn)
-      (void) ConcatenateString(&command," &");
-    /* Execute delegate.  */
-    status=SystemCommand(image_info->verbose,command);
+#if defined(POSIX)
+    {
+      MagickBool
+        needs_shell;
+
+      /*
+        Check to see if command template must be executed via shell
+        due to using constructs requiring multiple processes or I/O
+        redirection.
+      */
+      needs_shell = MagickFalse;
+      {
+        char *
+          p;
+
+        p = commands[i];
+        for (p = commands[i]; *p; p++)
+          {
+            if (('&' == *p) ||
+                (';' == *p) ||
+                ('<' == *p) ||
+                ('>' == *p) ||
+                ('|' == *p))
+              {
+                needs_shell = MagickTrue;
+                break;
+              }
+          }
+      }
+
+      if (MagickFalse == needs_shell)
+        {
+          int
+            arg_count,
+            j;
+          
+          char
+            **arg_array;
+          
+          /*
+            Convert command template into an argument array.  Translate
+            each argument array element individually in order to
+            absolutely avoid any possibility that the number of arguments
+            may be altered due to substituted data.
+          */
+          arg_array = StringToArgv(commands[i],&arg_count);
+          for (j = 0; arg_array[j] != (const char*) NULL; j++)
+            {
+              if (strchr(arg_array[j], '%') != (const char*) NULL)
+                {
+                  char *expanded = TranslateText(image_info,image,arg_array[j]);
+                  if (expanded != (char *) NULL)
+                    {
+                      MagickFreeMemory(arg_array[j]);
+                      arg_array[j] = expanded;
+                    }
+                }
+            }
+          
+          /*
+            Execute delegate using our secure "spawn" facility.
+          */
+          status = MagickSpawnVP(image_info->verbose,arg_array[1],arg_array+1);
+        }
+      else
+        {
+          /*
+            Expand sprintf-style codes in delegate command to command
+            string, escaping replacement text appropriately
+          */
+          command=TranslateTextEx(image_info,image,commands[i],UnixShellTextEscape);
+          if (command == (char *) NULL)
+            break;
+          /*
+            Execute delegate using command shell.
+          */
+          status=SystemCommand(image_info->verbose,command);
+        }
+    }
+#else
+    {
+      /*
+        Expand sprintf-style codes in delegate command to command string
+      */
+      command=TranslateText(image_info,image,commands[i]);
+      if (command == (char *) NULL)
+        break;
+      /*
+        Execute delegate using command shell.
+      */
+      status=SystemCommand(image_info->verbose,command);
+    }
+#endif
     MagickFreeMemory(command);
     /* Liberate convenience temporary files */
-    LiberateTemporaryFile(image_info->unique);
-    LiberateTemporaryFile(image_info->zero);
+    (void) LiberateTemporaryFile(image_info->unique);
+    (void) LiberateTemporaryFile(image_info->zero);
     if (status != False)
       {
         (void) ThrowException(exception,DelegateError,DelegateFailed,
@@ -536,12 +667,13 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
   */
  error_exit:
   if (temporary_image_filename)
-    LiberateTemporaryFile(image->filename);
+    (void) LiberateTemporaryFile(image->filename);
   for ( ; commands[i] != (char *) NULL; i++)
     MagickFreeMemory(commands[i]);
   MagickFreeMemory(commands);
   return(status != False);
 }
+
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -614,9 +746,16 @@ MagickExport unsigned int InvokePostscriptDelegate(const unsigned int verbose,
 #endif
   if (gs_func == (GhostscriptVectors *) NULL)
     {
-      if (verbose)
-        (void) fputs(command,stdout);
+#if defined(POSIX)
+      {
+        int arg_count;
+        char **arg_array;
+        arg_array = StringToArgv(command,&arg_count);
+        return MagickSpawnVP(verbose,arg_array[1],arg_array+1);
+      }
+#else
       return(SystemCommand(verbose,command));
+#endif
     }
   if (verbose)
     {
